/*
 * iaxshell - a minimalistic IAX2 shell
 *
 * Arnaldo M. Pereira - arnaldo@ansi-c.org
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <readline/readline.h>
#include <readline/history.h>

#include "iaxshell.h"

extern HIST_ENTRY **history_list ();

/*
 * Returns iaxshell commands that begins with 'text', or
 * NULL if there's none.
 */
char *iaxshell_command_generator (const char *text, int state)
{
	static int list_index, len;
	char *name;

	if (! state) {
		list_index = 0;
		len = strlen(text);
	}

	while ((name = allowed_cmds[list_index])) {
		list_index++;

		if (strncmp(name, text, len) == 0)
			return (strdup(name));
	}

	return NULL;
}

/*
 * Completes commands for readline
 */
char **iaxshell_completion (const char *text, int start, int end)
{
	char **matches;

	matches = (char **)NULL;

	if (start == 0)
		matches = rl_completion_matches (text, iaxshell_command_generator);

	return (matches);
}

/*
 * Initialize libreadline
 */
void iaxshell_init_readline (void)
{
	rl_readline_name = PROGRAM_NAME;
	rl_attempted_completion_function = iaxshell_completion;
}

/*
 * Read a line and save it on readline's history
 */
extern char *iaxshell_readline (void)
{
	char *l = readline(DEFAULT_PROMPT);
	
	if (l && strlen(l) > 1)
		add_history(l);

	return l;
}

/*
 * Show command history
 */
void iaxshell_show_history (void)
{
	HIST_ENTRY **list;
	register int i;

	list = history_list();
	if (list) {
		for (i = 0; list[i]; i++)
			fprintf (stderr, "%d: %s\r\n", i, list[i]->line);
	}

	return;
}
