/*
 * iaxshell - a minimalistic IAX2 shell
 *
 * Arnaldo M. Pereira - arnaldo@ansi-c.org
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>
#include <readline/readline.h>

#include "iaxclient.h"
#include "iaxshell.h"
#include "iaxreadline.h"

int do_levels = 0;
static int registered = 0;
static unsigned char user[40], pass[40], host[80];

/*
 * debug
 */
int iaxshell_levels_callback (float input, float output)
{
	if (do_levels) {
		fprintf(stderr, "IN: %f OUT: %f\n", input, output);
	}
	return 0;
}

/*
 * Call-state callback
 */
int iaxshell_state_callback (struct iaxc_ev_call_state call)
{
	if (call.state & IAXC_CALL_STATE_FREE) {

	} else if (call.state & IAXC_CALL_STATE_ACTIVE) {

	} else if (call.state & IAXC_CALL_STATE_OUTGOING) {

	} else if (call.state & IAXC_CALL_STATE_RINGING) {
		iaxc_select_call(call.callNo);

	} else if (call.state & IAXC_CALL_STATE_COMPLETE) {

	} else if (call.state & IAXC_CALL_STATE_SELECTED) {

	} else if (call.state & IAXC_CALL_STATE_BUSY) {

	} else if (call.state & IAXC_CALL_STATE_TRANSFER) {

	}

	return 0;
}

/*
 * debug
 */
int iaxshell_netstat_callback (struct iaxc_ev_netstats n)
{
	static int i;
	if(i++%25 == 0)
		fprintf(stderr, "RTT\t"
				"Rjit\tRlos%%\tRlosC\tRpkts\tRdel\tRdrop\tRooo\t"
				"Ljit\tLlos%%\tLlosC\tLpkts\tLdel\tLdrop\tLooo\n"
			   );

	fprintf(stderr, "%d\t"
			"%d\t%d\t%d\t%d\t%d\t%d\t%d\t"
			"%d\t%d\t%d\t%d\t%d\t%d\t%d\n",

			n.rtt,

			n.remote.jitter,
			n.remote.losspct,
			n.remote.losscnt,
			n.remote.packets,
			n.remote.delay,
			n.remote.dropped,
			n.remote.ooo,

			n.local.jitter,
			n.local.losspct,
			n.local.losscnt,
			n.local.packets,
			n.local.delay,
			n.local.dropped,
			n.local.ooo
		   );

	return 0;
}

/*
 * Registration callback
 */
int iaxshell_reg_callback (struct iaxc_ev_registration reg)
{
	switch (reg.reply) {
		case IAXC_REGISTRATION_REPLY_ACK:
			fprintf(stdout, "Registration accepted.\n");
			registered = 1;
			break;
		case IAXC_REGISTRATION_REPLY_REJ:
			fprintf(stdout, "Registration rejected.\n");
			break;
		case IAXC_REGISTRATION_REPLY_TIMEOUT:
			fprintf(stdout, "Registration timed-out.\n");
			break;
	}

	return 0;
}

/*
 * Text callback
 */
int iaxshell_text_callback (struct iaxc_ev_text text)
{
	switch (text.type) {
		case IAXC_TEXT_TYPE_STATUS:
			break;
		case IAXC_TEXT_TYPE_NOTICE:
			break;
		case IAXC_TEXT_TYPE_ERROR:
			break;
		case IAXC_TEXT_TYPE_FATALERROR:
			break;
		case IAXC_TEXT_TYPE_IAX:
			break;
	}

	fprintf(stderr, "[DEBUG] TEXT MESSAGE RECEIVED !\n");
	fprintf(stderr, "[DEBUG] type: %d\n", text.type);
	fprintf(stderr, "[DEBUG] callNo: %d\n", text.callNo);
	fprintf(stderr, "[DEBUG] message: %s\n", text.message);

	return 0;
}

/*
 * URL callback
 */
int iaxshell_url_callback (struct iaxc_ev_url url)
{
	switch (url.type) {
		case IAXC_URL_URL:
			break;
		case IAXC_URL_LDCOMPLETE:
			break;
		case IAXC_URL_LINKURL:
			break;
		case IAXC_URL_LINKREJECT:
			break;
		case IAXC_URL_UNLINK:
			break;
	}

	fprintf(stderr, "[DEBUG] URL RECEIVED !\n");
	fprintf(stderr, "[DEBUG] callNo: %d\n", url.callNo);
	fprintf(stderr, "[DEBUG] type: %d\n", url.type);
	fprintf(stderr, "[DEBUG] URL: %s\n", url.url);

	return 0;
}

/*
 * Video callback
 */
int iaxshell_video_callback (struct iaxc_ev_video video)
{
	fprintf(stderr, "[DEBUG] VIDEO RECEIVED !\n");
	fprintf(stderr, "[DEBUG] callNo: %d\n", video.callNo);
	fprintf(stderr, "[DEBUG] format: %d\n", video.format);
	fprintf(stderr, "[DEBUG] width: %d\n", video.width);
	fprintf(stderr, "[DEBUG] height: %d\n", video.height);
	fprintf(stderr, "[DEBUG] data ignored.\n");

	return 0;
}

/*
 * IAXclient callback
 * This is a general callback function, which just calls 
 * the proper ones
 */
int iaxc_callback(iaxc_event e)
{
	switch(e.type) {
		case IAXC_EVENT_TEXT:
			return iaxshell_text_callback(e.ev.text);
		case IAXC_EVENT_LEVELS:
			return iaxshell_levels_callback(e.ev.levels.input, e.ev.levels.output);
		case IAXC_EVENT_STATE:
			return iaxshell_state_callback(e.ev.call);
		case IAXC_EVENT_NETSTAT:
			return iaxshell_netstat_callback(e.ev.netstats);
		case IAXC_EVENT_URL:
			return iaxshell_url_callback(e.ev.url);
		case IAXC_EVENT_VIDEO:
			return iaxshell_video_callback(e.ev.video);
		case IAXC_EVENT_REGISTRATION:
			return iaxshell_reg_callback(e.ev.reg);

		default:
			return 0;
	}
}

/*
 * iaxshell initializer
 *	audio_type: One of the AUDIO_* constants from iaxclient.h
 *	number_calls: Number of calls iaxshell may handle simultaneously
 *	preferred_codecs: ORed bitmask of the preferred codecs
 *	allowed_codecs: ORed bitmask of the allowed codecs
 *	callback: iaxclient callback
 */
static int iaxshell_init (int audio_type, int number_calls, int preferred_codecs, 
		int allowed_codecs, iaxc_event_callback_t callback)
{
	/* int video_mode; */

	if(iaxc_initialize(audio_type, number_calls)) {
		fprintf(stderr, "iaxc_initialize() failed.\n");
		return 1;
	}

	iaxc_set_formats(preferred_codecs, allowed_codecs);
	iaxc_set_silence_threshold(0);  /* mute input */
	iaxc_set_event_callback(callback); 

	iaxc_start_processing_thread();

	iaxshell_init_readline();

#if 0
	video_mode = iaxc_video_mode_get();

	fprintf(stderr, "Video mode %d - ", video_mode);
	switch (video_mode) {
		case IAXC_VIDEO_MODE_NONE:
			fprintf(stderr, "don't send video at all.\n");
			break;
		case IAXC_VIDEO_MODE_ACTIVE:
			fprintf(stderr, "send video.\n");
			break;
		case IAXC_VIDEO_MODE_PREVIEW_RAW:
			fprintf(stderr, "send video and show RAW preview.\n");
			break;
		case IAXC_VIDEO_MODE_PREVIEW_ENCODED:
			fprintf(stderr, "send video and show encoded preview.\n");
			break;
	}
#endif

	return 0;
}

/*
 * Shuts iaxclient down and exit
 */
static void iaxshell_finish (void)
{
	iaxc_shutdown();
	exit(EXIT_SUCCESS);
}

/*
 * Signal handler that has to be written
 */
static void sighandler (int sig)
{
	switch (sig) {
		case SIGINT:
		case SIGTERM:
		case SIGHUP:
		default:
			fprintf(stderr, "Received signal %d, exiting.\n", sig);
			iaxshell_finish();
			break;
	}
}

int main(int argc, char **argv)
{
	int retval;

	signal(SIGINT,  sighandler);
	signal(SIGTERM, sighandler);
	signal(SIGHUP,  sighandler);

	/* slowly, the abstraction begins... */
	iaxshell_init(AUDIO_INTERNAL_PA, 1, IAXC_FORMAT_GSM, IAXC_FORMAT_GSM, iaxc_callback);

	for(;;) {
		unsigned char *line;
		unsigned char *p = NULL;

		line = iaxshell_readline();
		if (line == NULL)
			break;

		if ((p = strtok(line, " "))) {
			int i;

			for (i = 0; allowed_cmds[i] != NULL && p != NULL; i++) {

				if ((strcmp(p, allowed_cmds[i])) == 0) {
					switch (i) {

						/* TODO: multiple profiles support */
						case CMD_DIAL:
							if ((p = strtok(NULL, " "))) {
								unsigned char dialstr[64];
								if (registered)
									snprintf(dialstr, sizeof(dialstr), "%s/%s", host, p);
								else
									snprintf(dialstr, sizeof(dialstr), "%s", p);
								iaxc_call(dialstr);
								iaxc_set_silence_threshold(1);  /* unmute input */
							} else
								fprintf(stderr, "Usage: %s [user:pass@host/]extension\n", allowed_cmds[i]);
							break;

						case CMD_HANG:
							if ((p = strtok(NULL, " ")))
								fprintf(stderr, "Parameter `%s` ignored.\n", p);
							iaxc_dump_call();
							break;

						case CMD_SEND_DTMF:
							if ((p = strtok(NULL, " "))) {
								iaxc_send_dtmf(*p);
							} else
								fprintf(stderr, "Usage: %s digit\n", allowed_cmds[i]);
							break;

						case CMD_SEND_TEXT:

							if ((p = strtok(NULL, "\""))) {
								iaxc_send_text(p);
							} else
								fprintf(stderr, "Usage: %s \"string\"\n", allowed_cmds[i]);

							break;

						case CMD_REGISTER:
							if ((p = strtok(NULL, " "))) {

								unsigned char buf[80];

								memset(user, 0, sizeof(user));
								memset(pass, 0, sizeof(pass));
								memset(host, 0, sizeof(host));
								memset(buf, 0, sizeof(buf));

								memcpy(buf, p, (strlen(p) < sizeof(buf) ? strlen(p) : sizeof(buf)));
								if ((p = strtok(buf, ":"))) {
									snprintf(user, sizeof(user), p);
									if ((p = strtok(NULL, "@")) == NULL)
										break;
									snprintf(pass, sizeof(pass), p);
									if ((p = strtok(NULL, ":")) == NULL)
										break;
									snprintf(host, sizeof(host), p);
								}
								retval = iaxc_register(user, pass, host);
								if (retval != 1) {
									fprintf(stderr, "Failed to register\n");
									fprintf(stderr, "Tried with: %s:%s@%s\n", user, pass, host);
								} else {
									fprintf(stderr, "Registration for user [%s] sent, awaiting confirmation.\n", user);
								}

							} else
								fprintf(stderr, "Usage: %s user pass host\n", allowed_cmds[i]);
							break;

						case CMD_ANSWER:
							iaxc_set_audio_output(0);
							iaxc_answer_call(0);
							break;

						case CMD_HISTORY:
							iaxshell_show_history();
							break;

						default: 
							fprintf(stderr, "%s: command not found.\n", p);
							break;
					}
					break;
				}
			}
		}
	}

	printf("\n");
	iaxshell_finish();

	return EXIT_SUCCESS;
}
